﻿using System;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;

namespace MVVMLightToolkit.ViewModel
{



    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class ExampleViewModel : ViewModelBase
    {
        private Random rand = new Random();

        #region SecretNumber Property
        /// <summary>
        /// The <see cref="SecretNumber" /> property's name.
        /// </summary>
        public const string SecretNumberPropertyName = "SecretNumber";

        private int _secretNumber = 0;

        /// <summary>
        /// Gets the SecretNumber property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public int SecretNumber
        {
            get
            {
                return _secretNumber;
            }

            set
            {
                if (_secretNumber == value)
                    return;

                _secretNumber = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(SecretNumberPropertyName);
            }
        } 
        #endregion

        #region Suggestion Property
        /// <summary>
        /// The <see cref="Sugestion" /> property's name.
        /// </summary>
        public const string SuggestionPropertyName = "Suggestion";

        private string _suggestion = "";

        /// <summary>
        /// Gets the Sugestion property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string Suggestion
        {
            get
            {
                return _suggestion;
            }

            set
            {
                if (_suggestion == value)
                    return;

                _suggestion = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(SuggestionPropertyName);
            }
        }
        #endregion

        #region IsInProgress Property
        /// <summary>
        /// The <see cref="IsInProgress" /> property's name.
        /// </summary>
        public const string IsInProgressPropertyName = "IsInProgress";

        private bool _isInProgress = false;

        /// <summary>
        /// Gets the IsInProgress property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsInProgress
        {
            get
            {
                return _isInProgress;
            }

            set
            {
                if (_isInProgress == value)
                    return;

                _isInProgress = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(IsInProgressPropertyName);
            }
        }
        #endregion

        #region IsSolved
        /// <summary>
        /// The <see cref="IsSolved" /> property's name.
        /// </summary>
        public const string IsSolvedPropertyName = "IsSolved";

        private bool _isSolved = false;

        /// <summary>
        /// Gets the IsSolved property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsSolved
        {
            get
            {
                return _isSolved;
            }

            set
            {
                if (_isSolved == value)
                    return;

                _isSolved = value;

                // Update bindings, no broadcast
                RaisePropertyChanged(IsSolvedPropertyName);
            }
        }
        #endregion

        #region StartGuessing
        private RelayCommand _startCommand;

        public RelayCommand Start
        {
            get {
                if (_startCommand == null)
                {
                    _startCommand = new RelayCommand(
                        () =>
                        {
                            SecretNumber = rand.Next(10) + 1;
                            IsSolved = false;
                            IsInProgress = true;
                            Suggestion = "";
                        }
                    );
                }
                return _startCommand; }
        }

        #endregion

        #region CheckGuess Command
        private RelayCommand<int> _checkGuess;

        public RelayCommand<int> CheckGuess
        {
            get {
                if (_checkGuess == null)
                    _checkGuess = new RelayCommand<int>(
                        x =>
                        {
                            if (x == _secretNumber)
                            {
                                IsInProgress = false;
                                IsSolved = true;
                            }
                            else if (x < _secretNumber)
                                Suggestion = "to low";
                            else
                                Suggestion = "to high";
                        });

                return _checkGuess; 
            }
        }
        #endregion

        /// <summary>
        /// Initializes a new instance of the ExampleViewModel class.
        /// </summary>
        public ExampleViewModel()
        {

            ////if (IsInDesignMode)
            ////{
            ////    // Code runs in Blend --> create design time data.
            ////}
            ////else
            ////{
            ////    // Code runs "for real": Connect to service, etc...
            ////}
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}